package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
)

type CrawlerExecutionFilter struct {
	CrawlerID  *CrawlerID
	IsError    *bool
	HasChanges *bool
}

func (f CrawlerExecutionFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	if f.CrawlerID != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"crawlerID": *f.CrawlerID}}})
	}

	if f.IsError != nil {
		if *f.IsError {
			pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"error": bson.M{"$neq": nil}}}})
		} else {
			pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"error": bson.M{"$eq": nil}}}})
		}
	}

	if f.HasChanges != nil {
		if *f.HasChanges {
			pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"changes": bson.M{"$gt": 0}}}})
		} else {
			pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"changes": bson.M{"$eq": 0}}}})
		}
	}

	return pipeline
}

func (f CrawlerExecutionFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "creationTime", ct.SortDESC, "_id", ct.SortDESC
}
